# -*- coding: utf-8 -*-
"""
Created on Mon Apr 06 19:12:27 2020
Calculates longitdue, latitude and altitude of any IRIDIUM-satellite
Calculates azimuth and elevation at a given location
Chcks, if satellite is withon beam areas of LPDA

@author: STEG
"""
#------------------------------------------------------------------------------

from pyorbital.orbital import Orbital
from datetime import datetime
import re
import time
import matplotlib.pyplot as plt 
import ephem
import math
import os.path
import serial
#import numpy as np

#------------------------------------------------------------------------------

mylat  = 47.2031 # ° North
mylon  = 8.75615  # ° East
myalt  = 0.430   # km asl
EleMin = 13.0  # 10
EleMax = 85.0
AziMin = 110.0 # 110
AziMax = 225.0 # 225
MaxRange  = 70 # (+/- value) depends on your SAT-rotor type, check data sheetmyazi = -(azi - 180.0) # conversion 0° ... 360° -> +/- 180°

path = 'C:/DiSEcQ/Iridium/'
iridiumfile1 = 'iridium.txt'
iridiumfile2 = 'iridium-NEXT.txt'

FrameInnerx = [AziMin, AziMin, AziMax, AziMax, AziMin]
FrameInnery = [EleMin, EleMax, EleMax, EleMin, EleMin]
FrameOuter  = [AziMin-10, AziMax+15, EleMin-8, EleMax+5]

MyComport = 'COM5' # check with device manager (C:\Windows\System32\devmgmt.msc)

#------------------------------------------------------------------------------

def Track(azi,ele):
    myele = -(ele - 45.0)
    myazi = -(azi - 180.0) # conversion 0° ... 360° -> +/- 180°

    if (abs(myazi) < 0.5): # avoid -0 in azimuth
        myazi = 0.
    if (abs(myele) < 0.5): # avoid -0 in elevation
            myele = 0.

    try:
        DiSEqC = serial.Serial(
             port     = MyComport,
             baudrate = 9600,
             bytesize = serial.EIGHTBITS,
             parity   = serial.PARITY_NONE,
             timeout  = 2)
             
        if (DiSEqC.isOpen()):
            #print ("Successfully connected to antenna tracker at: "+DiSEqC.portstr)
            cmd = 'max{:6.2f}\r'.format(MaxRange) 
            DiSEqC.write(cmd.encode()) # set MaxRange
                        
            cmd = 'ele{:6.2f}\r'.format(myele) 
            DiSEqC.write(cmd.encode()) # set elevation or declination

            cmd = 'azi{:6.2f}\r'.format(myazi) 
            DiSEqC.write(cmd.encode()) # set azimuth or hour angle
            
            DiSEqC.close()
        
    except IOError:
            print ("Problem communication with tracker. Check COM-port and cables/connectors!")
    
#------------------------------------------------------------------------------

def SunCoo():
    MyLocation = ephem.Observer()
    MyLocation.lon      = str(mylon)
    MyLocation.lat      = str(mylat)
    MyLocation.elev     = 1000*myalt
    MyLocation.temp     = 20  # °C
    MyLocation.pressure = 900 # mbar
        
    dt = datetime.now() # PC must run on UT or GPS-time
    MyLocation.date = '{:4d}/{:02d}/{:02d} {:02d}:{:02d}:{:02d}'.format(
       dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second)
    
    sun = ephem.Sun()
    sun.compute(MyLocation)
    azi = math.degrees(sun.az)
    ele = math.degrees(sun.alt)
    return (azi,ele)

#------------------------------------------------------------------------------

def MoonCoo():
    MyLocation = ephem.Observer()
    MyLocation.lon      = str(mylon)
    MyLocation.lat      = str(mylat)
    MyLocation.elev     = 1000*myalt
    MyLocation.temp     = 20  # °C
    MyLocation.pressure = 900 # mbar
        
    dt = datetime.now() # PC must run on UT or GPS-time
    MyLocation.date = '{:4d}/{:02d}/{:02d} {:02d}:{:02d}:{:02d}'.format(
       dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second)
    
    moon = ephem.Moon()
    moon.compute(MyLocation)
    azi = math.degrees(moon.az)
    ele = math.degrees(moon.alt)
    return (azi,ele)

#------------------------------------------------------------------------------

def saveme(satname,azisat,elesat,azisun,elesun):
        dat = datetime.now().strftime("%Y-%m-%d_%H:%M") # "%Y-%m-%d %H:%M:%S"
        filename = satname + '_' + dat + '.prn'
        if (os.path.exists(filename)):
            with open(filename, "a") as fp:   # Save x/y-data in file
                hms = datetime.now().strftime("%H:%M:%S") # "%Y-%m-%d %H:%M:%S"
                st = hms +',{:8.2f},'.format(azisat) + '{:8.2f},'.format(elesat) + '{:8.0f}'.format(azisun)
                fp.write(st+'\n') 
        else:
            with open(filename, "a") as fp:   # Save x/y-data in file
                fp.write('Weather data USB-Stick BME280 (eBay.com)\n') # write header information
                fp.write('Docs: http://www.dogratian.com/products/index.php/menu-sensors/menu-usb-pa-type-a-bmp085\n')
                fp.write('TIME [UT], Temp [C], RH[%], Pressure [hP]\n') # write header information

#------------------------------------------------------------------------------
# Search satellites and their coordinates

sats = []
styp = []
f = open(path + iridiumfile1, 'r')
for line in f:
    st = repr(line)
    if ('IRIDIUM' in st):
        r = re.compile("[ \t\n\r:]+")
        y = r.split(st)
        key = 'IRIDIUM ' + y[1] + ' ' + y[2]
        sats.append(key)
        styp.append(iridiumfile1)
f.close()

#------------------------------------------------------------------------------

f = open(path + iridiumfile2, 'r')
for line in f:
    st = repr(line)
    if ('IRIDIUM' in st):
        r = re.compile("[ \t\n\r:]+")
        y = r.split(st)
        key = 'IRIDIUM ' + y[1]          
        sats.append(key)
        styp.append(iridiumfile2)
f.close()

#------------------------------------------------------------------------------

N = len(sats)
number=0
plt.figure()
plt.axis(FrameOuter)
plt.grid(which='major', color='green', linestyle='-')
plt.grid(which='minor', color='blue', linestyle='--')
plt.plot(FrameInnerx,FrameInnery,'-g',linewidth=5)
plt.xlabel('Azimuth [$\degree$]',fontsize=15)
plt.ylabel('Elevation [$\degree$]',fontsize=15)
plt.title('Satellite trajectory searching ...',fontsize=16)
sazi,sele=SunCoo()
plt.plot(sazi,sele, 'o',color='yellow', markersize=20)
mazi,mele=MoonCoo()
plt.plot(mazi,mele, 'o',color='orange', markersize=20)
plt.pause(0.2)
plt.show()

try:
    while True:
        dtobj = datetime.utcnow()
        print ('Check: ',str(dtobj))
        time.sleep(0.5)
        for i in range(N):
            #print (sats[i])
            snpp = Orbital(sats[i], tle_file = path + styp[i])
            dtobj = datetime.utcnow()
            p = snpp.get_lonlatalt(dtobj)
            #print ('Satellite-postion: ',p)
            
            lon, lat, alt = snpp.get_lonlatalt(dtobj)
            #print ("{} UTC at {}E, {}N, {}km".format(dtobj, round(lon, 3), round(lat, 3), round(alt, 3)))
            
            koo =  snpp.get_observer_look(dtobj, mylon, mylat, myalt)
            
            azi = koo[0]
            ele = koo[1]
            nam = sats[i]
            
            if ((ele>EleMin) and (ele<EleMax) and (azi>AziMin) and (azi<AziMax)): # local limits (gaden, house, Etzel...)
#            if ((ele>FrameOuter[2]) and (ele<FrameOuter[3]) and (azi>FrameOuter[0]) and (azi<FrameOuter[1])): # local limits (gaden, house, Etzel...)
                print ('Satellite ' + nam + ' in range at AZI={:5.1f} ELE={:4.1f}'.format(azi,ele) + ' at ' + str(dtobj))
                if ((ele>FrameInnery[0]) and (ele<FrameInnery[1] and (azi>FrameInnerx[0]) and (azi<FrameInnerx[2]))):
                    plt.plot(azi,ele,'og')
                    # position antenna here
                    Track(azi,ele)
                else:
                    plt.plot(azi,ele,'or')
                    

                #dat = datetime.now().strftime("%Y-%m-%d_%H:%M") # "%Y-%m-%d %H:%M:%S"
                filename = nam + '.prn'
                if (os.path.exists(filename)):
                    with open(filename, "a") as fp:   # Save x/y-data in file
                        hms = datetime.now().strftime("%H:%M:%S") # "%Y-%m-%d %H:%M:%S"
                        st = hms +',{:8.2f},'.format(azi) + '{:8.2f},'.format(ele) + '{:8.2f},'.format(sazi)+ '{:8.2f}'.format(sele)
                        fp.write(st+'\n') 
                else:
                    with open(filename, "a") as fp:   # Save x/y-data in file
                        fp.write('Local coordinates of IRIDIUM satellite, observed in Freienbach, JN47je\n') # write header information
                        fp.write('TIME [UT], AZI(sat), ELE(sat), AZI(sun), ELE(sun)\n') # write header information

                plt.pause(0.1)
                break
#            else:
#                plt.plot(azi,ele,'.r')
#                plt.pause(0.2)

            
        plt.clf()        
        plt.axis(FrameOuter)
        plt.grid(which='major', color='green', linestyle='-')
        plt.grid(which='minor', color='blue', linestyle='--')
        plt.plot(FrameInnerx,FrameInnery,'-g',linewidth=5)
        plt.xlabel('Azimuth [$\degree$]',fontsize=15)
        plt.ylabel('Elevation [$\degree$]',fontsize=15)
        plt.title('Satellite trajectory '+nam,fontsize=16)

        #plt.text(120,90,nam,fontsize=15)
        plt.text(120,7,'Azi={:5.1f}'.format(azi))
        plt.text(160,7,'Ele={:5.1f}'.format(ele))
        
        sazi,sele=SunCoo()
        plt.plot(sazi,sele, 'o',color='yellow', markersize=20)
        mazi,mele=MoonCoo()
        plt.plot(mazi,mele, 'o',color='orange', markersize=20)
        plt.show()
         
except KeyboardInterrupt:
    pass

print ('finito')

#------------------------------------------------------------------------------

